# -*- octave -*-
#

## Computes the impulse response of a system A with initial state x1
## (=B), from time 1 to k1, then at k1 eliminates node v, and proceeds
## further to time k2.  The failure is "detected" in the sense that
## other nodes that had outgoing edges towards node v will remove those
## edges and therefore rebalance their outgoing links.
##
## A: state-transition matrix
##
## x1: initial state vector
##
## k1: length of the first phase of the impulse response
##
## k2: length of the second phase of the impulse response
##
## v: node killed at time k1
##
function Y = impulse_response_with_detected_failure(A, x1, k1, k2, v=1)
  n = size(A)(1);
  ##
  ## we first compute k1 steps of the impulse response.
  ##
  ##A
  Y = [ x1 ];
  k1 = floor(k1/2);
  for i = 1:k1
    x1 = A*x1;
    Y = [ Y x1 ];
    x1 = A*x1;
    Y = [ Y x1 ];
  endfor
  ##
  ## it's now time to "kill" node v.  We start by finding all the nodes
  ## that have outgoing edges into v because those will have to be
  ## rebalanced.
  ##
  v_input_set = find(A(v,:));
  ## 
  ## for all those nodes, we must renormalize their outgoing edges. Here
  ## we do it in the easiest way, that is:
  ##
  for j = v_input_set
    ## we renormalize by dividing each element in the column A_*j 
    ## by 1 - A_vj
    r = 1 - A(v,j);
    if r == 0
      printf("error: killing node %d makes node %d a dead end.\n", v, j);
    else
      A(:,j) = A(:,j)/r;
    endif
  endfor
  ## now we are ready to zero-out all input and output edges of v
  A(:,v) = zeros(n,1);
  A(v,:) = zeros(1,n);
  ## as a paranoia precaution, we also zero out v's state in x
  x1(v) = 0;
  ## and now we are ready restart with the impulse response
  ##A
  k2 = floor(k2/2);
  for i = 1:k2
    x1 = A*x1;
    Y = [ Y x1 ];
    x1 = A*x1;
    Y = [ Y x1 ];
  endfor
